### Not yet implemented.  Collecting ideas and code snippets until it can be built.

# 'phone_us': re.compile(r'((?:\D|^)([+]?1?)?[ -)(.]?(\d{3}[-\. ]??\d{3}[-\. ]??\d{4}|\(\d{3}\)\s*\d{3}[-\. ]??\d{4}|\d{3}[-\. ]??\d{4})(?:\D|$))'),
        

# https://github.com/daviddrysdale/python-phonenumbers