# Not yet implemented.  Collecting ideas for eventual coding.

# Format of track data: https://www.magtek.com/content/documentationfiles/d99800004.pdf

# Maybe a working regex? https://stackoverflow.com/questions/2540865/regular-expression-for-credit-card-track-data 